@@ -1,6 +1,7 @@
-import numpy as np
 from io import BytesIO
 
+import numpy as np
+
 from matplotlib.testing.decorators import image_comparison
 import matplotlib.pyplot as plt
 import matplotlib.path as mpath
@@ -8,8 +9,8 @@ import matplotlib.patches as mpatches
 from matplotlib.ticker import FuncFormatter
 
 
-@image_comparison(baseline_images=['bbox_inches_tight'], remove_text=True,
-                  savefig_kwarg=dict(bbox_inches='tight'))
+@image_comparison(['bbox_inches_tight'], remove_text=True,
+                  savefig_kwarg={'bbox_inches': 'tight'})
 def test_bbox_inches_tight():
     #: Test that a figure saved using bbox_inches='tight' is clipped correctly
     data = [[66386, 174296, 75131, 577908, 32015],
@@ -32,15 +33,16 @@ def test_bbox_inches_tight():
         yoff = yoff + data[row]
         cellText.append([''])
     plt.xticks([])
+    plt.xlim(0, 5)
     plt.legend([''] * 5, loc=(1.2, 0.2))
+    fig.legend([''] * 5, bbox_to_anchor=(0, 0.2), loc='lower left')
     # Add a table at the bottom of the axes
     cellText.reverse()
-    the_table = plt.table(cellText=cellText,
-                          rowLabels=rowLabels,
-                          colLabels=colLabels, loc='bottom')
+    plt.table(cellText=cellText, rowLabels=rowLabels, colLabels=colLabels,
+              loc='bottom')
 
 
-@image_comparison(baseline_images=['bbox_inches_tight_suptile_legend'],
+@image_comparison(['bbox_inches_tight_suptile_legend'],
                   remove_text=False, savefig_kwarg={'bbox_inches': 'tight'})
 def test_bbox_inches_tight_suptile_legend():
     plt.plot(np.arange(10), label='a straight line')
@@ -59,7 +61,7 @@ def test_bbox_inches_tight_suptile_legend():
     plt.xlabel('X axis')
 
 
-@image_comparison(baseline_images=['bbox_inches_tight_clipping'],
+@image_comparison(['bbox_inches_tight_clipping'],
                   remove_text=True, savefig_kwarg={'bbox_inches': 'tight'})
 def test_bbox_inches_tight_clipping():
     # tests bbox clipping on scatter points, and path clipping on a patch
@@ -80,7 +82,7 @@ def test_bbox_inches_tight_clipping():
     plt.gcf().artists.append(patch)
 
 
-@image_comparison(baseline_images=['bbox_inches_tight_raster'],
+@image_comparison(['bbox_inches_tight_raster'],
                   remove_text=True, savefig_kwarg={'bbox_inches': 'tight'})
 def test_bbox_inches_tight_raster():
     """Test rasterization with tight_layout"""
@@ -90,9 +92,44 @@ def test_bbox_inches_tight_raster():
 
 
 def test_only_on_non_finite_bbox():
-
     fig, ax = plt.subplots()
     ax.annotate("", xy=(0, float('nan')))
     ax.set_axis_off()
     # we only need to test that it does not error out on save
     fig.savefig(BytesIO(), bbox_inches='tight', format='png')
+
+
+def test_tight_pcolorfast():
+    fig, ax = plt.subplots()
+    ax.pcolorfast(np.arange(4).reshape((2, 2)))
+    ax.set(ylim=(0, .1))
+    buf = BytesIO()
+    fig.savefig(buf, bbox_inches="tight")
+    buf.seek(0)
+    height, width, _ = plt.imread(buf).shape
+    # Previously, the bbox would include the area of the image clipped out by
+    # the axes, resulting in a very tall image given the y limits of (0, 0.1).
+    assert width > height
+
+
+def test_noop_tight_bbox():
+    from PIL import Image
+    x_size, y_size = (10, 7)
+    dpi = 100
+    # make the figure just the right size up front
+    fig = plt.figure(frameon=False, dpi=dpi, figsize=(x_size/dpi, y_size/dpi))
+    ax = plt.Axes(fig, [0., 0., 1., 1.])
+    fig.add_axes(ax)
+    ax.set_axis_off()
+    ax.get_xaxis().set_visible(False)
+    ax.get_yaxis().set_visible(False)
+
+    data = np.arange(x_size * y_size).reshape(y_size, x_size)
+    ax.imshow(data)
+    out = BytesIO()
+    fig.savefig(out, bbox_inches='tight', pad_inches=0)
+    out.seek(0)
+    im = np.asarray(Image.open(out))
+    assert (im[:, :, 3] == 255).all()
+    assert not (im[:, :, :3] == 255).all()
+    assert im.shape == (7, 10, 4)
