@@ -1,26 +1,16 @@
 import datetime
-import tempfile
-from unittest.mock import Mock
 
 import dateutil.tz
 import dateutil.rrule
+import functools
 import numpy as np
 import pytest
 
-from matplotlib.testing.decorators import image_comparison
-import matplotlib.pyplot as plt
-from matplotlib.cbook import MatplotlibDeprecationWarning
-import matplotlib.dates as mdates
-import matplotlib.ticker as mticker
 from matplotlib import rc_context
-
-
-def __has_pytz():
-    try:
-        import pytz
-        return True
-    except ImportError:
-        return False
+import matplotlib.dates as mdates
+import matplotlib.pyplot as plt
+from matplotlib.testing.decorators import image_comparison
+import matplotlib.ticker as mticker
 
 
 def test_date_numpyx():
@@ -33,12 +23,12 @@ def test_date_numpyx():
     ax = fig.add_subplot(1, 1, 1)
     h, = ax.plot(time, data)
     hnp, = ax.plot(timenp, data)
-    assert np.array_equal(h.get_xdata(orig=False), hnp.get_xdata(orig=False))
+    np.testing.assert_equal(h.get_xdata(orig=False), hnp.get_xdata(orig=False))
     fig = plt.figure(figsize=(10, 2))
     ax = fig.add_subplot(1, 1, 1)
     h, = ax.plot(data, time)
     hnp, = ax.plot(data, timenp)
-    assert np.array_equal(h.get_ydata(orig=False), hnp.get_ydata(orig=False))
+    np.testing.assert_equal(h.get_ydata(orig=False), hnp.get_ydata(orig=False))
 
 
 @pytest.mark.parametrize('t0', [datetime.datetime(2017, 1, 1, 0, 1, 1),
@@ -51,20 +41,20 @@ def test_date_numpyx():
                                  [datetime.datetime(2017, 1, 1, 2, 1, 1),
                                   datetime.datetime(2017, 1, 1, 3, 1, 1)]]])
 @pytest.mark.parametrize('dtype', ['datetime64[s]',
-                                    'datetime64[us]',
-                                    'datetime64[ms]',
-                                    'datetime64[ns]'])
+                                   'datetime64[us]',
+                                   'datetime64[ms]',
+                                   'datetime64[ns]'])
 def test_date_date2num_numpy(t0, dtype):
     time = mdates.date2num(t0)
     tnp = np.array(t0, dtype=dtype)
     nptime = mdates.date2num(tnp)
-    assert np.array_equal(time, nptime)
+    np.testing.assert_equal(time, nptime)
 
 
 @pytest.mark.parametrize('dtype', ['datetime64[s]',
-                                    'datetime64[us]',
-                                    'datetime64[ms]',
-                                    'datetime64[ns]'])
+                                   'datetime64[us]',
+                                   'datetime64[ms]',
+                                   'datetime64[ns]'])
 def test_date2num_NaT(dtype):
     t0 = datetime.datetime(2017, 1, 1, 0, 1, 1)
     tmpl = [mdates.date2num(t0), np.nan]
@@ -79,9 +69,9 @@ def test_date2num_NaT_scalar(units):
     assert np.isnan(tmpl)
 
 
-@image_comparison(baseline_images=['date_empty'], extensions=['png'])
+@image_comparison(['date_empty.png'])
 def test_date_empty():
-    # make sure mpl does the right thing when told to plot dates even
+    # make sure we do the right thing when told to plot dates even
     # if no date data has been presented, cf
     # http://sourceforge.net/tracker/?func=detail&aid=2850075&group_id=80706&atid=560720
     fig = plt.figure()
@@ -89,7 +79,7 @@ def test_date_empty():
     ax.xaxis_date()
 
 
-@image_comparison(baseline_images=['date_axhspan'], extensions=['png'])
+@image_comparison(['date_axhspan.png'])
 def test_date_axhspan():
     # test ax hspan with date inputs
     t0 = datetime.datetime(2009, 1, 20)
@@ -102,7 +92,7 @@ def test_date_axhspan():
     fig.subplots_adjust(left=0.25)
 
 
-@image_comparison(baseline_images=['date_axvspan'], extensions=['png'])
+@image_comparison(['date_axvspan.png'])
 def test_date_axvspan():
     # test ax hspan with date inputs
     t0 = datetime.datetime(2000, 1, 20)
@@ -115,8 +105,7 @@ def test_date_axvspan():
     fig.autofmt_xdate()
 
 
-@image_comparison(baseline_images=['date_axhline'],
-                  extensions=['png'])
+@image_comparison(['date_axhline.png'])
 def test_date_axhline():
     # test ax hline with date inputs
     t0 = datetime.datetime(2009, 1, 20)
@@ -129,8 +118,7 @@ def test_date_axhline():
     fig.subplots_adjust(left=0.25)
 
 
-@image_comparison(baseline_images=['date_axvline'],
-                  extensions=['png'])
+@image_comparison(['date_axvline.png'])
 def test_date_axvline():
     # test ax hline with date inputs
     t0 = datetime.datetime(2000, 1, 20)
@@ -143,12 +131,13 @@ def test_date_axvline():
     fig.autofmt_xdate()
 
 
-def test_too_many_date_ticks():
+def test_too_many_date_ticks(caplog):
     # Attempt to test SF 2715172, see
     # https://sourceforge.net/tracker/?func=detail&aid=2715172&group_id=80706&atid=560720
     # setting equal datetimes triggers and expander call in
     # transforms.nonsingular which results in too many ticks in the
-    # DayLocator.  This should trigger a Locator.MAXTICKS RuntimeError
+    # DayLocator.  This should emit a log at WARNING level.
+    caplog.set_level("WARNING")
     t0 = datetime.datetime(2000, 1, 20)
     tf = datetime.datetime(2000, 1, 20)
     fig = plt.figure()
@@ -156,18 +145,35 @@ def test_too_many_date_ticks():
     with pytest.warns(UserWarning) as rec:
         ax.set_xlim((t0, tf), auto=True)
         assert len(rec) == 1
-        assert 'Attempting to set identical left==right' in str(rec[0].message)
+        assert \
+            'Attempting to set identical left == right' in str(rec[0].message)
     ax.plot([], [])
     ax.xaxis.set_major_locator(mdates.DayLocator())
-    with pytest.raises(RuntimeError):
-        fig.savefig('junk.png')
+    v = ax.xaxis.get_major_locator()()
+    assert len(v) > 1000
+    # The warning is emitted multiple times because the major locator is also
+    # called both when placing the minor ticks (for overstriking detection) and
+    # during tick label positioning.
+    assert caplog.records and all(
+        record.name == "matplotlib.ticker" and record.levelname == "WARNING"
+        for record in caplog.records)
+    assert len(caplog.records) > 0
 
 
-@image_comparison(baseline_images=['RRuleLocator_bounds'], extensions=['png'])
+def _new_epoch_decorator(thefunc):
+    @functools.wraps(thefunc)
+    def wrapper():
+        mdates._reset_epoch_test_example()
+        mdates.set_epoch('2000-01-01')
+        thefunc()
+        mdates._reset_epoch_test_example()
+    return wrapper
+
+
+@image_comparison(['RRuleLocator_bounds.png'])
 def test_RRuleLocator():
     import matplotlib.testing.jpl_units as units
     units.register()
-
     # This will cause the RRuleLocator to go out of bounds when it tries
     # to add padding to the limits, so we make sure it caps at the correct
     # boundary values.
@@ -196,8 +202,7 @@ def test_RRuleLocator_dayrange():
     # On success, no overflow error shall be thrown
 
 
-@image_comparison(baseline_images=['DateFormatter_fractionalSeconds'],
-                  extensions=['png'])
+@image_comparison(['DateFormatter_fractionalSeconds.png'])
 def test_DateFormatter():
     import matplotlib.testing.jpl_units as units
     units.register()
@@ -222,62 +227,47 @@ def test_DateFormatter():
     fig.autofmt_xdate()
 
 
-def test_date_formatter_strftime():
+def test_locator_set_formatter():
     """
-    Tests that DateFormatter matches datetime.strftime,
-    check microseconds for years before 1900 for bug #3179
-    as well as a few related issues for years before 1900.
+    Test if setting the locator only will update the AutoDateFormatter to use
+    the new locator.
     """
-    def test_strftime_fields(dt):
-        """For datetime object dt, check DateFormatter fields"""
-        # Note: the last couple of %%s are to check multiple %s are handled
-        # properly; %% should get replaced by %.
-        formatter = mdates.DateFormatter("%w %d %m %y %Y %H %I %M %S %%%f %%x")
-        # Compute date fields without using datetime.strftime,
-        # since datetime.strftime does not work before year 1900
-        formatted_date_str = (
-            "{weekday} {day:02d} {month:02d} {year:02d} {full_year:04d} "
-            "{hour24:02d} {hour12:02d} {minute:02d} {second:02d} "
-            "%{microsecond:06d} %x"
-            .format(
-                weekday=str((dt.weekday() + 1) % 7),
-                day=dt.day,
-                month=dt.month,
-                year=dt.year % 100,
-                full_year=dt.year,
-                hour24=dt.hour,
-                hour12=((dt.hour-1) % 12) + 1,
-                minute=dt.minute,
-                second=dt.second,
-                microsecond=dt.microsecond))
-        with pytest.warns(MatplotlibDeprecationWarning):
-            assert formatter.strftime(dt) == formatted_date_str
+    plt.rcParams["date.autoformatter.minute"] = "%d %H:%M"
+    t = [datetime.datetime(2018, 9, 30, 8, 0),
+         datetime.datetime(2018, 9, 30, 8, 59),
+         datetime.datetime(2018, 9, 30, 10, 30)]
+    x = [2, 3, 1]
 
-        try:
-            # Test strftime("%x") with the current locale.
-            import locale  # Might not exist on some platforms, such as Windows
-            locale_formatter = mdates.DateFormatter("%x")
-            locale_d_fmt = locale.nl_langinfo(locale.D_FMT)
-            expanded_formatter = mdates.DateFormatter(locale_d_fmt)
-            with pytest.warns(MatplotlibDeprecationWarning):
-                assert locale_formatter.strftime(dt) == \
-                    expanded_formatter.strftime(dt)
-        except (ImportError, AttributeError):
-            pass
+    fig, ax = plt.subplots()
+    ax.plot(t, x)
+    ax.xaxis.set_major_locator(mdates.MinuteLocator((0, 30)))
+    fig.canvas.draw()
+    ticklabels = [tl.get_text() for tl in ax.get_xticklabels()]
+    expected = ['30 08:00', '30 08:30', '30 09:00',
+                '30 09:30', '30 10:00', '30 10:30']
+    assert ticklabels == expected
 
-    for year in range(1, 3000, 71):
-        # Iterate through random set of years
-        test_strftime_fields(datetime.datetime(year, 1, 1))
-        test_strftime_fields(datetime.datetime(year, 2, 3, 4, 5, 6, 12345))
+    ax.xaxis.set_major_locator(mticker.NullLocator())
+    ax.xaxis.set_minor_locator(mdates.MinuteLocator((5, 55)))
+    decoy_loc = mdates.MinuteLocator((12, 27))
+    ax.xaxis.set_minor_formatter(mdates.AutoDateFormatter(decoy_loc))
+
+    ax.xaxis.set_minor_locator(mdates.MinuteLocator((15, 45)))
+    fig.canvas.draw()
+    ticklabels = [tl.get_text() for tl in ax.get_xticklabels(which="minor")]
+    expected = ['30 08:15', '30 08:45', '30 09:15', '30 09:45', '30 10:15']
+    assert ticklabels == expected
 
 
 def test_date_formatter_callable():
-    scale = -11
-    locator = Mock(_get_unit=Mock(return_value=scale))
-    callable_formatting_function = (lambda dates, _:
-                                    [dt.strftime('%d-%m//%Y') for dt in dates])
 
-    formatter = mdates.AutoDateFormatter(locator)
+    class _Locator:
+        def _get_unit(self): return -11
+
+    def callable_formatting_function(dates, _):
+        return [dt.strftime('%d-%m//%Y') for dt in dates]
+
+    formatter = mdates.AutoDateFormatter(_Locator())
     formatter.scaled[-10] = callable_formatting_function
     assert formatter([datetime.datetime(2014, 12, 25)]) == ['25-12//2014']
 
@@ -310,26 +300,7 @@ def test_drange():
     assert mdates.num2date(daterange[-1]) == (end - delta)
 
 
-def test_empty_date_with_year_formatter():
-    # exposes sf bug 2861426:
-    # https://sourceforge.net/tracker/?func=detail&aid=2861426&group_id=80706&atid=560720
-
-    # update: I am no longer believe this is a bug, as I commented on
-    # the tracker.  The question is now: what to do with this test
-
-    import matplotlib.dates as dates
-
-    fig = plt.figure()
-    ax = fig.add_subplot(111)
-
-    yearFmt = dates.DateFormatter('%Y')
-    ax.xaxis.set_major_formatter(yearFmt)
-
-    with tempfile.TemporaryFile() as fh:
-        with pytest.raises(ValueError):
-            fig.savefig(fh)
-
-
+@_new_epoch_decorator
 def test_auto_date_locator():
     def _create_auto_date_locator(date1, date2):
         locator = mdates.AutoDateLocator(interval_multiples=False)
@@ -390,7 +361,8 @@ def test_auto_date_locator():
                  '1990-01-01 00:00:00+00:00',
                  '1990-01-01 00:00:00.000500+00:00',
                  '1990-01-01 00:00:00.001000+00:00',
-                 '1990-01-01 00:00:00.001500+00:00']
+                 '1990-01-01 00:00:00.001500+00:00',
+                 '1990-01-01 00:00:00.002000+00:00']
                 ],
                )
 
@@ -400,6 +372,7 @@ def test_auto_date_locator():
         assert list(map(str, mdates.num2date(locator()))) == expected
 
 
+@_new_epoch_decorator
 def test_auto_date_locator_intmult():
     def _create_auto_date_locator(date1, date2):
         locator = mdates.AutoDateLocator(interval_multiples=True)
@@ -464,7 +437,8 @@ def test_auto_date_locator_intmult():
                  '1997-01-01 00:00:00+00:00',
                  '1997-01-01 00:00:00.000500+00:00',
                  '1997-01-01 00:00:00.001000+00:00',
-                 '1997-01-01 00:00:00.001500+00:00']
+                 '1997-01-01 00:00:00.001500+00:00',
+                 '1997-01-01 00:00:00.002000+00:00']
                 ],
                )
 
@@ -475,6 +449,203 @@ def test_auto_date_locator_intmult():
         assert list(map(str, mdates.num2date(locator()))) == expected
 
 
+def test_concise_formatter():
+    def _create_auto_date_locator(date1, date2):
+        fig, ax = plt.subplots()
+
+        locator = mdates.AutoDateLocator(interval_multiples=True)
+        formatter = mdates.ConciseDateFormatter(locator)
+        ax.yaxis.set_major_locator(locator)
+        ax.yaxis.set_major_formatter(formatter)
+        ax.set_ylim(date1, date2)
+        fig.canvas.draw()
+        sts = [st.get_text() for st in ax.get_yticklabels()]
+        return sts
+
+    d1 = datetime.datetime(1997, 1, 1)
+    results = ([datetime.timedelta(weeks=52 * 200),
+                [str(t) for t in range(1980, 2201, 20)]
+                ],
+               [datetime.timedelta(weeks=52),
+                ['1997', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug',
+                 'Sep', 'Oct', 'Nov', 'Dec']
+                ],
+               [datetime.timedelta(days=141),
+                ['Jan', '22', 'Feb', '22', 'Mar', '22', 'Apr', '22',
+                 'May', '22']
+                ],
+               [datetime.timedelta(days=40),
+                ['Jan', '05', '09', '13', '17', '21', '25', '29', 'Feb',
+                 '05', '09']
+                ],
+               [datetime.timedelta(hours=40),
+                ['Jan-01', '04:00', '08:00', '12:00', '16:00', '20:00',
+                 'Jan-02', '04:00', '08:00', '12:00', '16:00']
+                ],
+               [datetime.timedelta(minutes=20),
+                ['00:00', '00:05', '00:10', '00:15', '00:20']
+                ],
+               [datetime.timedelta(seconds=40),
+                ['00:00', '05', '10', '15', '20', '25', '30', '35', '40']
+                ],
+               [datetime.timedelta(seconds=2),
+                ['59.5', '00:00', '00.5', '01.0', '01.5', '02.0', '02.5']
+                ],
+               )
+    for t_delta, expected in results:
+        d2 = d1 + t_delta
+        strings = _create_auto_date_locator(d1, d2)
+        assert strings == expected
+
+
+def test_concise_formatter_formats():
+    formats = ['%Y', '%m/%Y', 'day: %d',
+               '%H hr %M min', '%H hr %M min', '%S.%f sec']
+
+    def _create_auto_date_locator(date1, date2):
+        fig, ax = plt.subplots()
+
+        locator = mdates.AutoDateLocator(interval_multiples=True)
+        formatter = mdates.ConciseDateFormatter(locator, formats=formats)
+        ax.yaxis.set_major_locator(locator)
+        ax.yaxis.set_major_formatter(formatter)
+        ax.set_ylim(date1, date2)
+        fig.canvas.draw()
+        sts = [st.get_text() for st in ax.get_yticklabels()]
+        return sts
+
+    d1 = datetime.datetime(1997, 1, 1)
+    results = (
+        [datetime.timedelta(weeks=52 * 200), [str(t) for t in range(1980,
+         2201, 20)]],
+        [datetime.timedelta(weeks=52), [
+            '1997', '02/1997', '03/1997', '04/1997', '05/1997', '06/1997',
+            '07/1997', '08/1997', '09/1997', '10/1997', '11/1997', '12/1997',
+            ]],
+        [datetime.timedelta(days=141), [
+            '01/1997', 'day: 22', '02/1997', 'day: 22', '03/1997', 'day: 22',
+            '04/1997', 'day: 22', '05/1997', 'day: 22',
+            ]],
+        [datetime.timedelta(days=40), [
+            '01/1997', 'day: 05', 'day: 09', 'day: 13', 'day: 17', 'day: 21',
+            'day: 25', 'day: 29', '02/1997', 'day: 05', 'day: 09',
+            ]],
+        [datetime.timedelta(hours=40), [
+            'day: 01', '04 hr 00 min', '08 hr 00 min', '12 hr 00 min',
+            '16 hr 00 min', '20 hr 00 min', 'day: 02', '04 hr 00 min',
+            '08 hr 00 min', '12 hr 00 min', '16 hr 00 min',
+            ]],
+        [datetime.timedelta(minutes=20), ['00 hr 00 min', '00 hr 05 min',
+         '00 hr 10 min', '00 hr 15 min', '00 hr 20 min']],
+        [datetime.timedelta(seconds=40), [
+            '00 hr 00 min', '05.000000 sec', '10.000000 sec',
+            '15.000000 sec', '20.000000 sec', '25.000000 sec',
+            '30.000000 sec', '35.000000 sec', '40.000000 sec',
+            ]],
+        [datetime.timedelta(seconds=2), [
+            '59.500000 sec', '00 hr 00 min', '00.500000 sec', '01.000000 sec',
+            '01.500000 sec', '02.000000 sec', '02.500000 sec',
+            ]],
+        )
+    for t_delta, expected in results:
+        d2 = d1 + t_delta
+        strings = _create_auto_date_locator(d1, d2)
+        assert strings == expected
+
+
+def test_concise_formatter_zformats():
+    zero_formats = ['', "'%y", '%B', '%m-%d', '%S', '%S.%f']
+
+    def _create_auto_date_locator(date1, date2):
+        fig, ax = plt.subplots()
+
+        locator = mdates.AutoDateLocator(interval_multiples=True)
+        formatter = mdates.ConciseDateFormatter(
+            locator, zero_formats=zero_formats)
+        ax.yaxis.set_major_locator(locator)
+        ax.yaxis.set_major_formatter(formatter)
+        ax.set_ylim(date1, date2)
+        fig.canvas.draw()
+        sts = [st.get_text() for st in ax.get_yticklabels()]
+        return sts
+
+    d1 = datetime.datetime(1997, 1, 1)
+    results = ([datetime.timedelta(weeks=52 * 200),
+                [str(t) for t in range(1980, 2201, 20)]
+                ],
+               [datetime.timedelta(weeks=52),
+                ["'97", 'Feb', 'Mar', 'Apr', 'May', 'Jun',
+                    'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec']
+                ],
+               [datetime.timedelta(days=141),
+                ['January', '22', 'February', '22', 'March',
+                    '22', 'April', '22', 'May', '22']
+                ],
+               [datetime.timedelta(days=40),
+                ['January', '05', '09', '13', '17', '21',
+                    '25', '29', 'February', '05', '09']
+                ],
+               [datetime.timedelta(hours=40),
+                ['01-01', '04:00', '08:00', '12:00', '16:00', '20:00',
+                    '01-02', '04:00', '08:00', '12:00', '16:00']
+                ],
+               [datetime.timedelta(minutes=20),
+                ['00', '00:05', '00:10', '00:15', '00:20']
+                ],
+               [datetime.timedelta(seconds=40),
+                ['00', '05', '10', '15', '20', '25', '30', '35', '40']
+                ],
+               [datetime.timedelta(seconds=2),
+                ['59.5', '00.0', '00.5', '01.0', '01.5', '02.0', '02.5']
+                ],
+               )
+    for t_delta, expected in results:
+        d2 = d1 + t_delta
+        strings = _create_auto_date_locator(d1, d2)
+        assert strings == expected
+
+
+def test_concise_formatter_tz():
+    def _create_auto_date_locator(date1, date2, tz):
+        fig, ax = plt.subplots()
+
+        locator = mdates.AutoDateLocator(interval_multiples=True)
+        formatter = mdates.ConciseDateFormatter(locator, tz=tz)
+        ax.yaxis.set_major_locator(locator)
+        ax.yaxis.set_major_formatter(formatter)
+        ax.set_ylim(date1, date2)
+        fig.canvas.draw()
+        sts = [st.get_text() for st in ax.get_yticklabels()]
+        return sts, ax.yaxis.get_offset_text().get_text()
+
+    d1 = datetime.datetime(1997, 1, 1).replace(tzinfo=datetime.timezone.utc)
+    results = ([datetime.timedelta(hours=40),
+                ['03:00', '07:00', '11:00', '15:00', '19:00', '23:00',
+                 '03:00', '07:00', '11:00', '15:00', '19:00'],
+                "1997-Jan-02"
+                ],
+               [datetime.timedelta(minutes=20),
+                ['03:00', '03:05', '03:10', '03:15', '03:20'],
+                "1997-Jan-01"
+                ],
+               [datetime.timedelta(seconds=40),
+                ['03:00', '05', '10', '15', '20', '25', '30', '35', '40'],
+                "1997-Jan-01 03:00"
+                ],
+               [datetime.timedelta(seconds=2),
+                ['59.5', '03:00', '00.5', '01.0', '01.5', '02.0', '02.5'],
+                "1997-Jan-01 03:00"
+                ],
+               )
+
+    new_tz = datetime.timezone(datetime.timedelta(hours=3))
+    for t_delta, expected_strings, expected_offset in results:
+        d2 = d1 + t_delta
+        strings, offset = _create_auto_date_locator(d1, d2, new_tz)
+        assert strings == expected_strings
+        assert offset == expected_offset
+
+
 def test_auto_date_locator_intmult_tz():
     def _create_auto_date_locator(date1, date2, tz):
         locator = mdates.AutoDateLocator(interval_multiples=True, tz=tz)
@@ -546,8 +717,7 @@ def test_auto_date_locator_intmult_tz():
             assert st == expected
 
 
-@image_comparison(baseline_images=['date_inverted_limit'],
-                  extensions=['png'])
+@image_comparison(['date_inverted_limit.png'])
 def test_date_inverted_limit():
     # test ax hline with date inputs
     t0 = datetime.datetime(2009, 1, 20)
@@ -563,6 +733,7 @@ def test_date_inverted_limit():
 
 def _test_date2num_dst(date_range, tz_convert):
     # Timezones
+
     BRUSSELS = dateutil.tz.gettz('Europe/Brussels')
     UTC = mdates.UTC
 
@@ -575,8 +746,8 @@ def _test_date2num_dst(date_range, tz_convert):
 
     dt_utc = date_range(start=dtstart, freq=interval, periods=N)
     dt_bxl = tz_convert(dt_utc, BRUSSELS)
-
-    expected_ordinalf = [735322.0 + (i * interval_days) for i in range(N)]
+    t0 = 735322.0 + mdates.date2num(np.datetime64('0000-12-31'))
+    expected_ordinalf = [t0 + (i * interval_days) for i in range(N)]
     actual_ordinalf = list(mdates.date2num(dt_bxl))
 
     assert actual_ordinalf == expected_ordinalf
@@ -593,6 +764,7 @@ def test_date2num_dst():
         datetime-like object that applies timezone normalization after
         subtraction.
         """
+
         def __sub__(self, other):
             r = super().__sub__(other)
             tzinfo = getattr(r, 'tzinfo', None)
@@ -679,10 +851,9 @@ def test_rrulewrapper():
 
 
 @pytest.mark.pytz
-@pytest.mark.skipif(not __has_pytz(), reason="Requires pytz")
 def test_rrulewrapper_pytz():
     # Test to make sure pytz zones are supported in rrules
-    import pytz
+    pytz = pytest.importorskip("pytz")
 
     def attach_tz(dt, zi):
         return zi.localize(dt)
@@ -691,21 +862,21 @@ def test_rrulewrapper_pytz():
 
 
 @pytest.mark.pytz
-@pytest.mark.skipif(not __has_pytz(), reason="Requires pytz")
 def test_yearlocator_pytz():
-    import pytz
+    pytz = pytest.importorskip("pytz")
 
     tz = pytz.timezone('America/New_York')
     x = [tz.localize(datetime.datetime(2010, 1, 1))
-            + datetime.timedelta(i) for i in range(2000)]
+         + datetime.timedelta(i) for i in range(2000)]
     locator = mdates.AutoDateLocator(interval_multiples=True, tz=tz)
     locator.create_dummy_axis()
     locator.set_view_interval(mdates.date2num(x[0])-1.0,
                               mdates.date2num(x[-1])+1.0)
-
-    np.testing.assert_allclose([733408.208333, 733773.208333, 734138.208333,
-                                734503.208333, 734869.208333,
-                                735234.208333, 735599.208333], locator())
+    t = np.array([733408.208333, 733773.208333, 734138.208333,
+                  734503.208333, 734869.208333, 735234.208333, 735599.208333])
+    # convert to new epoch from old...
+    t = t + mdates.date2num(np.datetime64('0000-12-31'))
+    np.testing.assert_allclose(t, locator())
     expected = ['2009-01-01 00:00:00-05:00',
                 '2010-01-01 00:00:00-05:00', '2011-01-01 00:00:00-05:00',
                 '2012-01-01 00:00:00-05:00', '2013-01-01 00:00:00-05:00',
@@ -743,4 +914,36 @@ def test_num2timedelta(x, tdelta):
 def test_datetime64_in_list():
     dt = [np.datetime64('2000-01-01'), np.datetime64('2001-01-01')]
     dn = mdates.date2num(dt)
-    assert np.array_equal(dn, [730120.,  730486.])
+    # convert fixed values from old to new epoch
+    t = (np.array([730120.,  730486.]) +
+         mdates.date2num(np.datetime64('0000-12-31')))
+    np.testing.assert_equal(dn, t)
+
+
+def test_change_epoch():
+    date = np.datetime64('2000-01-01')
+
+    with pytest.raises(RuntimeError):
+        # this should fail here because there is a sentinel on the epoch
+        # if the epoch has been used then it cannot be set.
+        mdates.set_epoch('0000-01-01')
+
+    # use private method to clear the epoch and allow it to be set...
+    mdates._reset_epoch_test_example()
+    mdates.set_epoch('1970-01-01')
+    dt = (date - np.datetime64('1970-01-01')).astype('datetime64[D]')
+    dt = dt.astype('int')
+    np.testing.assert_equal(mdates.date2num(date), float(dt))
+
+    mdates._reset_epoch_test_example()
+    mdates.set_epoch('0000-12-31')
+    np.testing.assert_equal(mdates.date2num(date), 730120.0)
+
+    mdates._reset_epoch_test_example()
+    mdates.set_epoch('1970-01-01T01:00:00')
+    np.testing.assert_allclose(mdates.date2num(date), dt - 1./24.)
+    mdates._reset_epoch_test_example()
+    mdates.set_epoch('1970-01-01T00:00:00')
+    np.testing.assert_allclose(
+        mdates.date2num(np.datetime64('1970-01-01T12:00:00')),
+        0.5)
