@@ -1,18 +1,22 @@
+import functools
+
 import pytest
 
 from mpl_toolkits.mplot3d import Axes3D, axes3d, proj3d, art3d
 import matplotlib as mpl
 from matplotlib import cm
-from matplotlib import path as mpath
 from matplotlib import colors as mcolors
 from matplotlib.testing.decorators import image_comparison, check_figures_equal
-from matplotlib.cbook.deprecation import MatplotlibDeprecationWarning
 from matplotlib.collections import LineCollection, PolyCollection
 from matplotlib.patches import Circle
 import matplotlib.pyplot as plt
 import numpy as np
 
 
+mpl3d_image_comparison = functools.partial(
+    image_comparison, remove_text=True, style='default')
+
+
 def test_aspect_equal_error():
     fig = plt.figure()
     ax = fig.add_subplot(111, projection='3d')
@@ -20,7 +24,7 @@ def test_aspect_equal_error():
         ax.set_aspect('equal')
 
 
-@image_comparison(['bar3d.png'], remove_text=True)
+@mpl3d_image_comparison(['bar3d.png'])
 def test_bar3d():
     fig = plt.figure()
     ax = fig.add_subplot(111, projection='3d')
@@ -32,13 +36,25 @@ def test_bar3d():
         ax.bar(xs, ys, zs=z, zdir='y', align='edge', color=cs, alpha=0.8)
 
 
-@image_comparison(['bar3d_shaded.png'], remove_text=True)
+def test_bar3d_colors():
+    fig = plt.figure()
+    ax = fig.add_subplot(111, projection='3d')
+    for c in ['red', 'green', 'blue', 'yellow']:
+        xs = np.arange(len(c))
+        ys = np.zeros_like(xs)
+        zs = np.zeros_like(ys)
+        # Color names with same length as xs/ys/zs should not be split into
+        # individual letters.
+        ax.bar3d(xs, ys, zs, 1, 1, 1, color=c)
+
+
+@mpl3d_image_comparison(['bar3d_shaded.png'])
 def test_bar3d_shaded():
     x = np.arange(4)
     y = np.arange(5)
     x2d, y2d = np.meshgrid(x, y)
     x2d, y2d = x2d.ravel(), y2d.ravel()
-    z = x2d + y2d
+    z = x2d + y2d + 1  # Avoid triggering bug with zero-depth boxes.
 
     views = [(-60, 30), (30, 30), (30, -30), (120, -30)]
     fig = plt.figure(figsize=plt.figaspect(1 / len(views)))
@@ -52,7 +68,7 @@ def test_bar3d_shaded():
     fig.canvas.draw()
 
 
-@image_comparison(['bar3d_notshaded.png'], remove_text=True)
+@mpl3d_image_comparison(['bar3d_notshaded.png'])
 def test_bar3d_notshaded():
     fig = plt.figure()
     ax = fig.add_subplot(111, projection='3d')
@@ -92,7 +108,7 @@ def test_bar3d_lightsource():
     np.testing.assert_array_equal(color, collection._facecolors3d[1::6])
 
 
-@image_comparison(['contour3d.png'], remove_text=True, style='mpl20')
+@mpl3d_image_comparison(['contour3d.png'])
 def test_contour3d():
     fig = plt.figure()
     ax = fig.gca(projection='3d')
@@ -105,7 +121,7 @@ def test_contour3d():
     ax.set_zlim(-100, 100)
 
 
-@image_comparison(['contourf3d.png'], remove_text=True)
+@mpl3d_image_comparison(['contourf3d.png'])
 def test_contourf3d():
     fig = plt.figure()
     ax = fig.gca(projection='3d')
@@ -118,7 +134,7 @@ def test_contourf3d():
     ax.set_zlim(-100, 100)
 
 
-@image_comparison(['contourf3d_fill.png'], remove_text=True)
+@mpl3d_image_comparison(['contourf3d_fill.png'])
 def test_contourf3d_fill():
     fig = plt.figure()
     ax = fig.gca(projection='3d')
@@ -133,7 +149,7 @@ def test_contourf3d_fill():
     ax.set_zlim(-1, 1)
 
 
-@image_comparison(['tricontour.png'], remove_text=True, style='mpl20')
+@mpl3d_image_comparison(['tricontour.png'])
 def test_tricontour():
     fig = plt.figure()
 
@@ -148,7 +164,7 @@ def test_tricontour():
     ax.tricontourf(x, y, z)
 
 
-@image_comparison(['lines3d.png'], remove_text=True)
+@mpl3d_image_comparison(['lines3d.png'])
 def test_lines3d():
     fig = plt.figure()
     ax = fig.gca(projection='3d')
@@ -160,7 +176,15 @@ def test_lines3d():
     ax.plot(x, y, z)
 
 
-@image_comparison(['mixedsubplot.png'], remove_text=True)
+@check_figures_equal(extensions=["png"])
+def test_plot_scalar(fig_test, fig_ref):
+    ax1 = fig_test.gca(projection='3d')
+    ax1.plot([1], [1], "o")
+    ax2 = fig_ref.gca(projection='3d')
+    ax2.plot(1, 1, "o")
+
+
+@mpl3d_image_comparison(['mixedsubplot.png'])
 def test_mixedsubplots():
     def f(t):
         return np.cos(2*np.pi*t) * np.exp(-t)
@@ -197,17 +221,18 @@ def test_tight_layout_text(fig_test, fig_ref):
     ax2.text(.5, .5, .5, s='some string')
 
 
-@image_comparison(['scatter3d.png'], remove_text=True)
+@mpl3d_image_comparison(['scatter3d.png'])
 def test_scatter3d():
     fig = plt.figure()
     ax = fig.add_subplot(111, projection='3d')
     ax.scatter(np.arange(10), np.arange(10), np.arange(10),
                c='r', marker='o')
-    ax.scatter(np.arange(10, 20), np.arange(10, 20), np.arange(10, 20),
-               c='b', marker='^')
+    x = y = z = np.arange(10, 20)
+    ax.scatter(x, y, z, c='b', marker='^')
+    z[-1] = 0  # Check that scatter() copies the data.
 
 
-@image_comparison(['scatter3d_color.png'], remove_text=True)
+@mpl3d_image_comparison(['scatter3d_color.png'])
 def test_scatter3d_color():
     fig = plt.figure()
     ax = fig.add_subplot(111, projection='3d')
@@ -217,7 +242,52 @@ def test_scatter3d_color():
                color='b', marker='s')
 
 
-@image_comparison(['plot_3d_from_2d.png'], remove_text=True)
+@pytest.mark.parametrize('azim', [-50, 130])  # yellow first, blue first
+@check_figures_equal(extensions=['png'])
+def test_marker_draw_order_data_reversed(fig_test, fig_ref, azim):
+    """
+    Test that the draw order does not depend on the data point order.
+
+    For the given viewing angle at azim=-50, the yellow marker should be in
+    front. For azim=130, the blue marker should be in front.
+    """
+    x = [-1, 1]
+    y = [1, -1]
+    z = [0, 0]
+    color = ['b', 'y']
+    ax = fig_test.add_subplot(projection='3d')
+    ax.scatter(x, y, z, s=3500, c=color)
+    ax.view_init(elev=0, azim=azim)
+    ax = fig_ref.add_subplot(projection='3d')
+    ax.scatter(x[::-1], y[::-1], z[::-1], s=3500, c=color[::-1])
+    ax.view_init(elev=0, azim=azim)
+
+
+@check_figures_equal(extensions=['png'])
+def test_marker_draw_order_view_rotated(fig_test, fig_ref):
+    """
+    Test that the draw order changes with the direction.
+
+    If we rotate *azim* by 180 degrees and exchange the colors, the plot
+    plot should look the same again.
+    """
+    azim = 130
+    x = [-1, 1]
+    y = [1, -1]
+    z = [0, 0]
+    color = ['b', 'y']
+    ax = fig_test.add_subplot(projection='3d')
+    # axis are not exactly invariant under 180 degree rotation -> deactivate
+    ax.set_axis_off()
+    ax.scatter(x, y, z, s=3500, c=color)
+    ax.view_init(elev=0, azim=azim)
+    ax = fig_ref.add_subplot(projection='3d')
+    ax.set_axis_off()
+    ax.scatter(x, y, z, s=3500, c=color[::-1])  # color reversed
+    ax.view_init(elev=0, azim=azim - 180)  # view rotated by 180 degrees
+
+
+@mpl3d_image_comparison(['plot_3d_from_2d.png'], tol=0.01)
 def test_plot_3d_from_2d():
     fig = plt.figure()
     ax = fig.add_subplot(111, projection='3d')
@@ -227,7 +297,7 @@ def test_plot_3d_from_2d():
     ax.plot(xs, ys, zs=0, zdir='y')
 
 
-@image_comparison(['surface3d.png'], remove_text=True)
+@mpl3d_image_comparison(['surface3d.png'])
 def test_surface3d():
     fig = plt.figure()
     ax = fig.gca(projection='3d')
@@ -242,7 +312,7 @@ def test_surface3d():
     fig.colorbar(surf, shrink=0.5, aspect=5)
 
 
-@image_comparison(['surface3d_shaded.png'], remove_text=True)
+@mpl3d_image_comparison(['surface3d_shaded.png'])
 def test_surface3d_shaded():
     fig = plt.figure()
     ax = fig.gca(projection='3d')
@@ -256,7 +326,7 @@ def test_surface3d_shaded():
     ax.set_zlim(-1.01, 1.01)
 
 
-@image_comparison(['text3d.png'])
+@mpl3d_image_comparison(['text3d.png'], remove_text=False)
 def test_text3d():
     fig = plt.figure()
     ax = fig.gca(projection='3d')
@@ -280,7 +350,7 @@ def test_text3d():
     ax.set_zlabel('Z axis')
 
 
-@image_comparison(['trisurf3d.png'], remove_text=True, tol=0.03)
+@mpl3d_image_comparison(['trisurf3d.png'], tol=0.03)
 def test_trisurf3d():
     n_angles = 36
     n_radii = 8
@@ -298,7 +368,7 @@ def test_trisurf3d():
     ax.plot_trisurf(x, y, z, cmap=cm.jet, linewidth=0.2)
 
 
-@image_comparison(['trisurf3d_shaded.png'], remove_text=True, tol=0.03)
+@mpl3d_image_comparison(['trisurf3d_shaded.png'], tol=0.03)
 def test_trisurf3d_shaded():
     n_angles = 36
     n_radii = 8
@@ -316,7 +386,7 @@ def test_trisurf3d_shaded():
     ax.plot_trisurf(x, y, z, color=[1, 0.5, 0], linewidth=0.2)
 
 
-@image_comparison(['wireframe3d.png'], remove_text=True)
+@mpl3d_image_comparison(['wireframe3d.png'])
 def test_wireframe3d():
     fig = plt.figure()
     ax = fig.add_subplot(111, projection='3d')
@@ -324,7 +394,7 @@ def test_wireframe3d():
     ax.plot_wireframe(X, Y, Z, rcount=13, ccount=13)
 
 
-@image_comparison(['wireframe3dzerocstride.png'], remove_text=True)
+@mpl3d_image_comparison(['wireframe3dzerocstride.png'])
 def test_wireframe3dzerocstride():
     fig = plt.figure()
     ax = fig.add_subplot(111, projection='3d')
@@ -332,7 +402,7 @@ def test_wireframe3dzerocstride():
     ax.plot_wireframe(X, Y, Z, rcount=13, ccount=0)
 
 
-@image_comparison(['wireframe3dzerorstride.png'], remove_text=True)
+@mpl3d_image_comparison(['wireframe3dzerorstride.png'])
 def test_wireframe3dzerorstride():
     fig = plt.figure()
     ax = fig.add_subplot(111, projection='3d')
@@ -358,37 +428,27 @@ def test_mixedsamplesraises():
         ax.plot_surface(X, Y, Z, cstride=50, rcount=10)
 
 
-@image_comparison(['quiver3d.png'], remove_text=True)
+@mpl3d_image_comparison(
+    ['quiver3d.png', 'quiver3d_pivot_middle.png', 'quiver3d_pivot_tail.png'])
 def test_quiver3d():
-    fig = plt.figure()
-    ax = fig.gca(projection='3d')
-
     x, y, z = np.ogrid[-1:0.8:10j, -1:0.8:10j, -1:0.6:3j]
-
     u = np.sin(np.pi * x) * np.cos(np.pi * y) * np.cos(np.pi * z)
     v = -np.cos(np.pi * x) * np.sin(np.pi * y) * np.cos(np.pi * z)
-    w = (np.sqrt(2.0 / 3.0) * np.cos(np.pi * x) * np.cos(np.pi * y) *
-            np.sin(np.pi * z))
+    w = (2/3)**0.5 * np.cos(np.pi * x) * np.cos(np.pi * y) * np.sin(np.pi * z)
+    for pivot in ['tip', 'middle', 'tail']:
+        ax = plt.figure().add_subplot(projection='3d')
+        ax.quiver(x, y, z, u, v, w, length=0.1, pivot=pivot, normalize=True)
 
+
+@check_figures_equal(extensions=["png"])
+def test_quiver3d_empty(fig_test, fig_ref):
+    fig_ref.add_subplot(projection='3d')
+    x = y = z = u = v = w = []
+    ax = fig_test.add_subplot(projection='3d')
     ax.quiver(x, y, z, u, v, w, length=0.1, pivot='tip', normalize=True)
 
 
-@image_comparison(['quiver3d_empty.png'], remove_text=True)
-def test_quiver3d_empty():
-    fig = plt.figure()
-    ax = fig.gca(projection='3d')
-
-    x, y, z = np.ogrid[-1:0.8:0j, -1:0.8:0j, -1:0.6:0j]
-
-    u = np.sin(np.pi * x) * np.cos(np.pi * y) * np.cos(np.pi * z)
-    v = -np.cos(np.pi * x) * np.sin(np.pi * y) * np.cos(np.pi * z)
-    w = (np.sqrt(2.0 / 3.0) * np.cos(np.pi * x) * np.cos(np.pi * y) *
-            np.sin(np.pi * z))
-
-    ax.quiver(x, y, z, u, v, w, length=0.1, pivot='tip', normalize=True)
-
-
-@image_comparison(['quiver3d_masked.png'], remove_text=True)
+@mpl3d_image_comparison(['quiver3d_masked.png'])
 def test_quiver3d_masked():
     fig = plt.figure()
     ax = fig.gca(projection='3d')
@@ -399,45 +459,14 @@ def test_quiver3d_masked():
 
     u = np.sin(np.pi * x) * np.cos(np.pi * y) * np.cos(np.pi * z)
     v = -np.cos(np.pi * x) * np.sin(np.pi * y) * np.cos(np.pi * z)
-    w = (np.sqrt(2.0 / 3.0) * np.cos(np.pi * x) * np.cos(np.pi * y) *
-            np.sin(np.pi * z))
+    w = (2/3)**0.5 * np.cos(np.pi * x) * np.cos(np.pi * y) * np.sin(np.pi * z)
     u = np.ma.masked_where((-0.4 < x) & (x < 0.1), u, copy=False)
     v = np.ma.masked_where((0.1 < y) & (y < 0.7), v, copy=False)
 
     ax.quiver(x, y, z, u, v, w, length=0.1, pivot='tip', normalize=True)
 
 
-@image_comparison(['quiver3d_pivot_middle.png'], remove_text=True)
-def test_quiver3d_pivot_middle():
-    fig = plt.figure()
-    ax = fig.gca(projection='3d')
-
-    x, y, z = np.ogrid[-1:0.8:10j, -1:0.8:10j, -1:0.6:3j]
-
-    u = np.sin(np.pi * x) * np.cos(np.pi * y) * np.cos(np.pi * z)
-    v = -np.cos(np.pi * x) * np.sin(np.pi * y) * np.cos(np.pi * z)
-    w = (np.sqrt(2.0 / 3.0) * np.cos(np.pi * x) * np.cos(np.pi * y) *
-            np.sin(np.pi * z))
-
-    ax.quiver(x, y, z, u, v, w, length=0.1, pivot='middle', normalize=True)
-
-
-@image_comparison(['quiver3d_pivot_tail.png'], remove_text=True)
-def test_quiver3d_pivot_tail():
-    fig = plt.figure()
-    ax = fig.gca(projection='3d')
-
-    x, y, z = np.ogrid[-1:0.8:10j, -1:0.8:10j, -1:0.6:3j]
-
-    u = np.sin(np.pi * x) * np.cos(np.pi * y) * np.cos(np.pi * z)
-    v = -np.cos(np.pi * x) * np.sin(np.pi * y) * np.cos(np.pi * z)
-    w = (np.sqrt(2.0 / 3.0) * np.cos(np.pi * x) * np.cos(np.pi * y) *
-            np.sin(np.pi * z))
-
-    ax.quiver(x, y, z, u, v, w, length=0.1, pivot='tail', normalize=True)
-
-
-@image_comparison(['poly3dcollection_closed.png'], remove_text=True)
+@mpl3d_image_comparison(['poly3dcollection_closed.png'])
 def test_poly3dcollection_closed():
     fig = plt.figure()
     ax = fig.gca(projection='3d')
@@ -459,7 +488,7 @@ def test_poly_collection_2d_to_3d_empty():
     assert poly.get_paths() == []
 
 
-@image_comparison(['poly3dcollection_alpha.png'], remove_text=True)
+@mpl3d_image_comparison(['poly3dcollection_alpha.png'])
 def test_poly3dcollection_alpha():
     fig = plt.figure()
     ax = fig.gca(projection='3d')
@@ -476,7 +505,7 @@ def test_poly3dcollection_alpha():
     ax.add_collection3d(c2)
 
 
-@image_comparison(['axes3d_labelpad.png'])
+@mpl3d_image_comparison(['axes3d_labelpad.png'], remove_text=False)
 def test_axes3d_labelpad():
     fig = plt.figure()
     ax = Axes3D(fig)
@@ -496,7 +525,7 @@ def test_axes3d_labelpad():
         tick.set_pad(tick.get_pad() - i * 5)
 
 
-@image_comparison(['axes3d_cla.png'])
+@mpl3d_image_comparison(['axes3d_cla.png'], remove_text=False)
 def test_axes3d_cla():
     # fixed in pull request 4553
     fig = plt.figure()
@@ -505,6 +534,13 @@ def test_axes3d_cla():
     ax.cla()  # make sure the axis displayed is 3D (not 2D)
 
 
+@mpl3d_image_comparison(['axes3d_rotated.png'], remove_text=False)
+def test_axes3d_rotated():
+    fig = plt.figure()
+    ax = fig.add_subplot(1, 1, 1, projection='3d')
+    ax.view_init(90, 45)  # look down, rotated. Should be square
+
+
 def test_plotsurface_1d_raises():
     x = np.linspace(0.5, 10, num=100)
     y = np.linspace(0.5, 10, num=100)
@@ -560,7 +596,7 @@ def _test_proj_draw_axes(M, s=1, *args, **kwargs):
     return fig, ax
 
 
-@image_comparison(['proj3d_axes_cube.png'], remove_text=True, style='default')
+@mpl3d_image_comparison(['proj3d_axes_cube.png'])
 def test_proj_axes_cube():
     M = _test_proj_make_M()
 
@@ -582,8 +618,7 @@ def test_proj_axes_cube():
     ax.set_ylim(-0.2, 0.2)
 
 
-@image_comparison(['proj3d_axes_cube_ortho.png'],
-                  remove_text=True, style='default')
+@mpl3d_image_comparison(['proj3d_axes_cube_ortho.png'])
 def test_proj_axes_cube_ortho():
     E = np.array([200, 100, 100])
     R = np.array([0, 0, 0])
@@ -632,8 +667,7 @@ def test_world():
                                 [0, 0, 0, 1]])
 
 
-@image_comparison(['proj3d_lines_dists.png'],
-                  remove_text=True, style='default')
+@mpl3d_image_comparison(['proj3d_lines_dists.png'])
 def test_lines_dists():
     fig, ax = plt.subplots(figsize=(4, 6), subplot_kw=dict(aspect='equal'))
 
@@ -667,7 +701,7 @@ def test_autoscale():
     assert ax.get_w_lims() == (0, 1, -.1, 1.1, -.4, 2.4)
 
 
-@image_comparison(['axes3d_ortho.png'], style='default')
+@mpl3d_image_comparison(['axes3d_ortho.png'], remove_text=False)
 def test_axes3d_ortho():
     fig = plt.figure()
     ax = fig.gca(projection='3d')
@@ -692,7 +726,7 @@ def test_invalid_axes_limits(setter, side, value):
 
 
 class TestVoxels:
-    @image_comparison(['voxels-simple.png'], remove_text=True)
+    @mpl3d_image_comparison(['voxels-simple.png'])
     def test_simple(self):
         fig, ax = plt.subplots(subplot_kw={"projection": "3d"})
 
@@ -700,8 +734,7 @@ class TestVoxels:
         voxels = (x == y) | (y == z)
         ax.voxels(voxels)
 
-    @image_comparison(['voxels-edge-style.png'],
-                      remove_text=True, style='default')
+    @mpl3d_image_comparison(['voxels-edge-style.png'])
     def test_edge_style(self):
         fig, ax = plt.subplots(subplot_kw={"projection": "3d"})
 
@@ -712,7 +745,7 @@ class TestVoxels:
         # change the edge color of one voxel
         v[max(v.keys())].set_edgecolor('C2')
 
-    @image_comparison(['voxels-named-colors.png'], remove_text=True)
+    @mpl3d_image_comparison(['voxels-named-colors.png'])
     def test_named_colors(self):
         """Test with colors set to a 3d object array of strings."""
         fig, ax = plt.subplots(subplot_kw={"projection": "3d"})
@@ -725,7 +758,7 @@ class TestVoxels:
         colors[(x + z) < 10] = 'cyan'
         ax.voxels(voxels, facecolors=colors)
 
-    @image_comparison(['voxels-rgb-data.png'], remove_text=True)
+    @mpl3d_image_comparison(['voxels-rgb-data.png'])
     def test_rgb_data(self):
         """Test with colors set to a 4d float array of rgb data."""
         fig, ax = plt.subplots(subplot_kw={"projection": "3d"})
@@ -738,7 +771,7 @@ class TestVoxels:
         colors[..., 2] = z / 9
         ax.voxels(voxels, facecolors=colors)
 
-    @image_comparison(['voxels-alpha.png'], remove_text=True)
+    @mpl3d_image_comparison(['voxels-alpha.png'])
     def test_alpha(self):
         fig, ax = plt.subplots(subplot_kw={"projection": "3d"})
 
@@ -756,7 +789,7 @@ class TestVoxels:
             assert voxels[coord], "faces returned for absent voxel"
             assert isinstance(poly, art3d.Poly3DCollection)
 
-    @image_comparison(['voxels-xyz.png'], tol=0.01)
+    @mpl3d_image_comparison(['voxels-xyz.png'], tol=0.01, remove_text=False)
     def test_xyz(self):
         fig, ax = plt.subplots(subplot_kw={"projection": "3d"})
 
@@ -803,13 +836,11 @@ class TestVoxels:
             ax.voxels(x, y, z, filled=filled, **kw)
 
         # duplicate argument
-        with pytest.raises(TypeError) as exc:
+        with pytest.raises(TypeError, match='voxels'):
             ax.voxels(x, y, z, filled, filled=filled)
-        exc.match(".*voxels.*")
         # missing arguments
-        with pytest.raises(TypeError) as exc:
+        with pytest.raises(TypeError, match='voxels'):
             ax.voxels(x, y)
-        exc.match(".*voxels.*")
         # x, y, z are positional only - this passes them on as attributes of
         # Poly3DCollection
         with pytest.raises(AttributeError):
@@ -860,61 +891,6 @@ def test_inverted_cla():
     assert not ax.zaxis_inverted()
 
 
-def test_art3d_deprecated():
-
-    with pytest.warns(MatplotlibDeprecationWarning):
-        art3d.norm_angle(0.0)
-
-    with pytest.warns(MatplotlibDeprecationWarning):
-        art3d.norm_text_angle(0.0)
-
-    path = mpath.Path(np.empty((0, 2)))
-
-    with pytest.warns(MatplotlibDeprecationWarning):
-        art3d.path_to_3d_segment(path)
-
-    with pytest.warns(MatplotlibDeprecationWarning):
-        art3d.paths_to_3d_segments([path])
-
-    with pytest.warns(MatplotlibDeprecationWarning):
-        art3d.path_to_3d_segment_with_codes(path)
-
-    with pytest.warns(MatplotlibDeprecationWarning):
-        art3d.paths_to_3d_segments_with_codes([path])
-
-    with pytest.warns(MatplotlibDeprecationWarning):
-        art3d.get_colors([], 1)
-
-    with pytest.warns(MatplotlibDeprecationWarning):
-        art3d.zalpha([], [])
-
-
-def test_proj3d_deprecated():
-    with pytest.warns(MatplotlibDeprecationWarning):
-        proj3d.line2d([0, 1], [0, 1])
-
-    with pytest.warns(MatplotlibDeprecationWarning):
-        proj3d.line2d_dist([0, 1, 3], [0, 1])
-
-    with pytest.warns(MatplotlibDeprecationWarning):
-        proj3d.mod([1, 1, 1])
-
-    vec = np.arange(4)
-    M = np.ones((4, 4))
-
-    with pytest.warns(MatplotlibDeprecationWarning):
-        proj3d.proj_transform_vec(vec, M)
-
-    with pytest.warns(MatplotlibDeprecationWarning):
-        proj3d.proj_transform_vec_clip(vec, M)
-
-    with pytest.warns(MatplotlibDeprecationWarning):
-        proj3d.vec_pad_ones(np.ones(3), np.ones(3), np.ones(3))
-
-    with pytest.warns(MatplotlibDeprecationWarning):
-        proj3d.proj_trans_clip_points(np.ones((4, 3)), M)
-
-
 def test_ax3d_tickcolour():
     fig = plt.figure()
     ax = Axes3D(fig)
@@ -986,3 +962,49 @@ def test_quiver3D_smoke(fig_test, fig_ref):
     for fig, length in zip((fig_ref, fig_test), (1, 1.0)):
         ax = fig.gca(projection="3d")
         ax.quiver(x, y, z, u, v, w, length=length, pivot=pivot)
+
+
+@image_comparison(["minor_ticks.png"], style="mpl20")
+def test_minor_ticks():
+    ax = plt.figure().add_subplot(projection="3d")
+    ax.set_xticks([0.25], minor=True)
+    ax.set_xticklabels(["quarter"], minor=True)
+    ax.set_yticks([0.33], minor=True)
+    ax.set_yticklabels(["third"], minor=True)
+    ax.set_zticks([0.50], minor=True)
+    ax.set_zticklabels(["half"], minor=True)
+
+
+@image_comparison(["equal_box_aspect.png"], style="mpl20")
+def test_equal_box_aspect():
+    from itertools import product, combinations
+
+    fig = plt.figure()
+    ax = fig.add_subplot(111, projection="3d")
+
+    # Make data
+    u = np.linspace(0, 2 * np.pi, 100)
+    v = np.linspace(0, np.pi, 100)
+    x = np.outer(np.cos(u), np.sin(v))
+    y = np.outer(np.sin(u), np.sin(v))
+    z = np.outer(np.ones_like(u), np.cos(v))
+
+    # Plot the surface
+    ax.plot_surface(x, y, z)
+
+    # draw cube
+    r = [-1, 1]
+    for s, e in combinations(np.array(list(product(r, r, r))), 2):
+        if np.sum(np.abs(s - e)) == r[1] - r[0]:
+            ax.plot3D(*zip(s, e), color="b")
+
+    # Make axes limits
+    xyzlim = np.column_stack(
+        [ax.get_xlim3d(), ax.get_ylim3d(), ax.get_zlim3d()]
+    )
+    XYZlim = [min(xyzlim[0]), max(xyzlim[1])]
+    ax.set_xlim3d(XYZlim)
+    ax.set_ylim3d(XYZlim)
+    ax.set_zlim3d(XYZlim)
+    ax.axis('off')
+    ax.set_box_aspect((1, 1, 1))
